/*
 * (C) Copyright 2018-2018, by John Sichi and Contributors.
 *
 * JGraphT : a free Java graph-theory library
 *
 * This program and the accompanying materials are dual-licensed under
 * either
 *
 * (a) the terms of the GNU Lesser General Public License version 2.1
 * as published by the Free Software Foundation, or (at your option) any
 * later version.
 *
 * or (per the licensee's choosing)
 *
 * (b) the terms of the Eclipse Public License v1.0 as published by
 * the Eclipse Foundation.
 */

package demo;

import org.jgrapht.*;
import org.jgrapht.graph.*;
import org.jgrapht.graph.builder.*;

/**
 * Demonstrates how to use {@link GraphTypeBuilder} and {@link GraphBuilder}.
 *
 * @author John Sichi
 */
public final class GraphBuilderDemo {
    public static void main(String [] args) {
       System.out.println("Enter GraphBuilderDemo.main() ... ");
       System.out.println("======================================= ... ");

       // Run buildKiteGraph() ...

       Graph<Integer, DefaultEdge> kite = buildKiteGraph();

       // Print graph ...

       System.out.println( kite.toString() );

       System.out.println("======================================= ... ");
       System.out.println("Enter GraphBuilderDemo.main() ... ");
    }

    /**
     * Builds a simple graph with no vertices.
     *
     * @return a modifiable empty graph instance
     */

    private static Graph<Integer, DefaultEdge> buildEmptySimpleGraph() {
        return GraphTypeBuilder.<Integer, DefaultEdge>undirected().
            allowingMultipleEdges(false).allowingSelfLoops(false).
            edgeClass(DefaultEdge.class).weighted(false).buildGraph();
    }
    //@example:buildType:end


    /**
     * Builds the <a href="http://mathworld.wolfram.com/KiteGraph.html">kite graph</a>.
     *
     * @return an unmodifiable instance of the kite graph
     */

    private static Graph<Integer, DefaultEdge> buildKiteGraph() {
        return new GraphBuilder<>(buildEmptySimpleGraph()).
            addEdgeChain(1, 2, 3, 4, 1).
            addEdge(2, 4).
            addEdge(3, 5).buildAsUnmodifiable();
    }
}

// End GraphBuilderDemo.java
