package demo.metro;

import javax.xml.bind.annotation.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import demo.metro.MetroBubble;
import demo.metro.Coordinate;

@XmlType(name = "MetroStationType",
         propOrder = {"stationName", "coord", "parking", "security", "busRoute", "wayPoint", "hint" })
public class MetroStation { 
    protected boolean parking  = false;
    protected boolean security = false;
    protected boolean busroute = false;
    protected boolean waypoint = true;
    protected List    onTrack;
    String  stationName;
    String      mapFile;
    @XmlElement(name="bubble")
    MetroBubble hint;
    @XmlElement(name="coordinate")
    Coordinate coord;

    // Contructor methods ...
    
    public MetroStation() {}

    public MetroStation( String name, double dX, double dY ) {
        stationName = name;
        onTrack = new ArrayList();  // Initialize array list of line colors
        coord   = new Coordinate (dX, dY);
        hint    = new MetroBubble();
    }

    public MetroStation( String name, double dX, double dY, boolean park, boolean sec, boolean br ) {
        stationName = name;
        onTrack = new ArrayList();  // Initialize array list of line colors
        coord   = new Coordinate (dX, dY);
        hint    = new MetroBubble();
         
        parking  = park;   
        security = sec;
        busroute = br;
    }
    
    public void setMapFile(String file){
        mapFile = file;
    }

    // Deal with bubble message ....
    
    public void setBubbleMessage ( String sMessage ) {
        hint.setMessage( sMessage );
    }

    // Deal with station name ....

    public void setStationName ( String sName ) {
        stationName = sName;
    }

    public String getStationName () {
        return stationName;
    }

    // Deal with station coordinates ....

    public void setCoord ( Coordinate c ) {
        coord = c;
    }

    public Coordinate findCoord () {
        return coord;
    }

    // Assign metro station to a track ....

    public void add( String track ) {
       onTrack.add ( track );
    }

    // Deal with bus route ...
    
    public void setWayPoint( boolean waypoint ) {
       this.waypoint = waypoint;
    }

    public boolean getWayPoint() {
        return waypoint;
    }

    // Deal with bus route ...
    
    public void setBusRoute( boolean busroute ) {
       this.busroute = busroute;
    }

    public boolean getBusRoute() {
        return busroute;
    }

    // Deal with security ...

    public void setSecurity( boolean security ) {
       this.security = security;
    }
    
    public boolean getSecurity() {
        return security;
    }

    // Deal with parking ...
    
    public void setParking( boolean park ) {
       parking = park;
    }
    
    public boolean getParking() {
        return parking;
    }

    // Convert description of metro station to a string ...

    public String toString() {
       String s = "MetroStation(\"" + stationName + "\")\n" +
                  "   Coordinates = (" + coord.getX() + "," + coord.getY() + ")\n" +
                  "   Parking = " + parking + "\n";

       // Walk along array list and add line names to string "s" ....

       if ( onTrack.size() > 0 ) {
          s = s.concat( "   Track   = { " );
          for (int i = 0; i < onTrack.size(); i = i + 1)
             s = s.concat( onTrack.get(i) + " ");
             s = s.concat("}\n");
         }

       return s;
   }
}
